# Captain ⛵️

<p align="center">
<img src="https://raw.githubusercontent.com/QuantumGray/captain/master/logo.jpeg">
</p>

<p align="center">
<a href="https://pub.dev/packages/captain"><img src="https://img.shields.io/pub/v/beamer.svg" alt="pub"></a>
<a href="https://github.com/QuantumGray/captain/blob/master/.github/workflows/test.yml"><img src="https://github.com/QuantumGray/captain/workflows/tests/badge.svg" alt="test"></a>
<a href="https://github.com/google/pedantic"><img src="https://dart-lang.github.io/linter/lints/style-pedantic.svg" alt="style"></a>
</p>

<p align="center">
<a href="https://github.com/QuantumGray/captain/commits/master"><img src="https://img.shields.io/github/commit-activity/m/QuantumGray/captain" alt="GitHub commit activity"></a>
<a href="https://github.com/QuantumGray/captain/issues"><img src="https://img.shields.io/github/issues-raw/QuantumGray/captain" alt="GitHub open issues"></a>
<a href="https://github.com/QuantumGray/captain/issues?q=is%3Aissue+is%3Aclosed"><img src="https://img.shields.io/github/issues-closed-raw/QuantumGray/captain" alt="GitHub closed issues"></a>
<a href="https://github.com/QuantumGray/captain/blob/master/LICENSE"><img src="https://img.shields.io/github/license/QuantumGray/captain" alt="Licence"></a>
</p>

### **an imperative way to navigate declaratively AKA making it dead simple for you to design complex navigations 🦾😎**

## 🌲 Placing it in the widget tree

place the `Captain()`widget at the same postition inside your widget tree where your Router or Navigator widget would reside. Best practice is placing the main Router beneath the Material/Cupertino/WidgetsApp like:

````dart
MaterialApp(
    home: Captain(
        CaptainConfig([
            PageToShowBeneath(),
            MyBeautifulHomePage(),
        ],),
        // only pages is required
    ),
),

````
### `CaptainConfig`

#### EXAMPLE
````dart
CaptainConfig(
    // REQUIRED
    [MyStartingPage()],
    // OPTIONAL
    popPage: (routeThatIsGettingPopped, itsResult, pageStack) => pageStack..removeLast(),
    shouldPop: (pageStack) => pageStack.isNotEmpty,
    actions: {
        'removeEverythingAndAddThePages': (pageStack) {
            pageStack.removeAll();
            return pageStack..addAll([
                Page1(),
                Page2(),
            ]);
        }
    }
    initialRouteInformation: RouteInformation(localtion: 'initial'),
    parseRouteInformation: (newRouteInformation, pageStack) {
        if (newRouteInformation.location=='/home' && pageStack.length < 2) {
            return HomePage();
        }
    },
    restoreRouteInformation: (page) {
        if (page is Home) {
            return RouteInformation(location: '/welcomeToHome');
        }
    }
)
````
| Parameter | Description | Docs |
| ----------- | ----------- | ----------- |
| `pages` | the stack of pages the app is built with at first | [pages](https://pub.dev/documentation/captain/latest/captain/CaptainConfig/pages.html) |
| `popPage` | function that is called when a page should be popped - this should usually remove the top-most page of the stack but can behave in any custom way | [popPage](https://pub.dev/documentation/captain/latest/captain/CaptainConfig/popPage.html) |
| `shouldPop` | decide on | [shouldPop](https://pub.dev/documentation/captain/latest/captain/CaptainConfig/shouldPop.html) |
| `actions` | map of preconfigured functions that are callable via `Navigator.of(context).action('key')` | [actions](https://pub.dev/documentation/captain/latest/captain/CaptainConfig/actions.html)|
| `initialRouteInformation` | initial RouteInformation object the app starts with | [initialRouteInformation]() |
| `parseRouteInformation` | function that maps a new incoming RouteInformation to a `Page` that is added on the stack | [parseRouteInformation](https://pub.dev/documentation/captain/latest/captain/CaptainConfig/parseRouteInformation.html) |
| `restoreRouteInformation` | chooses/restores new RouteInformation object taking the parsed page ☝️ as a parameter | [restoreRouteInformation](https://pub.dev/documentation/captain/latest/captain/CaptainConfig/restoreRouteInformation.html) |
---

## 🧭 Navigate with Captain

Captain supports imperative navigation style by complying to the `Navigator.of(context)` format

**USE EITHER:**
- `.action(Object actionKey)`for invoking predefined actions that have been registered to the Captain Widget
- `.actionFunc(List<Page> Function(List<Page>))` which takes a List<Page> and returns a List<Page> which will be the new Page stack for the App

### EXAMPLE

````dart
// invoke an action defined in actions parameter in CaptainConfig
Navigator.of(context).action("myActionKey");

// directly nvoke a change in the pageStack and return the new pageStack
Navigator.of(context).actionFunc((pageStack) => 
    pageStack..add(pageToAdd));

````